@@ -209,7 +209,7 @@
 (export lambda-list-keywords)
 
 (proclaim '(special *features*))
-(proclaim '(special compiler::*compiling*))
+(proclaim '(special compiler::*compiling* compiler::*compiling-from-file*))
 (setq compiler::*compiling* nil)
 
 (use-package '("COMMON-LISP" "EXT") "SYSTEM")
@@ -282,7 +282,8 @@
 (sys::%putd 'bye #'sys::%exit)
 
 (export
- '(re-export make-encoding encoding show-stack gc exit quit bye
+ '(re-export make-encoding encoding encoding-charset
+   show-stack gc exit quit bye
    probe-directory cd make-dir delete-dir default-directory dir
    xgcd exquo ! evalhook applyhook substring string-concat
    string-char make-char string-width char-width
@@ -318,7 +319,7 @@
 (common-lisp:in-package "CUSTOM")
 
 (common-lisp:export
- '(*load-paths* *editor* *clhs-root-default*
+ '(*load-paths* *editor* *clhs-root-default* *browsers* *browser*
    *load-echo* *applyhook* *evalhook* *load-compiling* *compile-warnings*
    ;; places.lisp
    *ansi* *current-language* *lib-directory* *default-file-encoding*
@@ -652,14 +653,14 @@
                ;; as PROGN, return a form, that returns without side-effects
                ;; the same values.
                ;; Else expand all arguments from the second one as forms.
-               (if (or (member 'COMPILE (second form))
-                       (member :COMPILE-TOPLEVEL (second form)))
-                 (values
-                  (list 'values-list
-                        (list 'quote
-                              (multiple-value-list
-                               (eval (cons 'PROGN (cddr form))))))
-                  t)
+               (if (member 'COMPILE (second form)) ; not :COMPILE-TOPLEVEL !
+                 (let ((compiler::*compiling-from-file* nil))
+                   (values
+                    (list 'values-list
+                          (list 'quote
+                                (multiple-value-list
+                                 (eval (cons 'PROGN (cddr form))))))
+                    t))
                  (multiple-value-call #'%expand-cons form
                    (first form) nil
                    (multiple-value-call #'%expand-cons (rest form)
@@ -831,7 +832,7 @@
                                          (cddr form))))
                           (do ((L3 (second form) (cdr L3)))
                               ((atom L3))
-                            (if (member (caar L3) specials :test #'eq)
+                            (if (memq (caar L3) specials)
                               (error-of-type 'source-program-error
                                 (TEXT "~S: symbol ~S must not be declared SPECIAL and a macro at the same time")
                                 'symbol-macrolet (caar L3)))))
@@ -985,7 +986,7 @@
         (%expand-parspez (first ll))
         (progn
           (let ((v (first ll)))
-            (if (not (member v lambda-list-keywords :test #'eq))
+            (if (not (memq v lambda-list-keywords))
               (setq *venv* (vector (%expand-varspec-var v) nil *venv*))))
           (%expand-lambdalist (rest ll))))))
 
@@ -1202,14 +1203,15 @@
 (proclaim '(special *load-paths*))
 (setq *load-paths* nil)
 (proclaim '(special *source-file-types*))
-(setq *source-file-types* '(#".lisp" #".lsp" #".cl"))
+(setq *source-file-types* '("lisp" "lsp" "cl"))
 (proclaim '(special *compiled-file-types*))
-(setq *compiled-file-types* '(#".fas"))
+(setq *compiled-file-types* '("fas"))
 
 ;; for the time being the files don't have to be searched:
 (defun search-file (filename extensions)
   (mapcan #'(lambda (extension)
-              (let ((filename (merge-pathnames filename extension)))
+              (let ((filename (merge-pathnames filename
+                                     (make-pathname :type extension))))
                 (if (probe-file filename) (list filename) '())))
           extensions))
 
@@ -1229,6 +1231,9 @@
 (setq *load-input-stream* nil)
 (proclaim '(special *load-level*))
 (setq *load-level* 0)
+#+ffi ; the default :language for DEF-CALL-* & C-FUNCTION -- see foreign1.lisp
+(proclaim '(special ffi::*foreign-language*))
+#+ffi (setq ffi::*foreign-language* nil)
 
 ;; (LOAD filename [:verbose] [:print] [:if-does-not-exist] [:external-format]
 ;;                [:echo] [:compiling] [:extra-file-types]),
@@ -1247,8 +1252,7 @@
                      #+UNICODE :external-format
                      #+UNICODE (if (member (pathname-type filename)
                                            *compiled-file-types*
-                                           :test #'string=
-                                           :key #'pathname-type)
+                                           :test #'string=)
                                    charset:utf-8
                                    external-format)
                      :if-does-not-exist nil)
@@ -1267,8 +1271,7 @@
                          #+UNICODE :external-format
                          #+UNICODE (if (member (pathname-type filename)
                                                *compiled-file-types*
-                                               :test #'string=
-                                               :key #'pathname-type)
+                                               :test #'string=)
                                        charset:utf-8
                                        external-format))))))))
     (if stream
@@ -1289,6 +1292,7 @@
              (*load-pathname* (if (pathnamep filename) filename nil))
              (*load-truename*
               (if (pathnamep filename) (truename filename) nil))
+             #+ffi (ffi::*foreign-language* ffi::*foreign-language*)
              (*package* *package*) ; bind *PACKAGE*
              (*readtable* *readtable*) ; bind *READTABLE*
              (end-of-file "EOF")) ; one-time Object
@@ -1320,9 +1324,8 @@
           (fresh-line)
           (write-string ";;")
           (write-string indent)
-          (write-string (TEXT "Loading of file "))
-          (princ filename)
-          (write-string (TEXT " is finished.")))
+          (write-string (TEXT "Loaded file "))
+          (princ filename))
         t)
       (if if-does-not-exist
         (error-of-type 'file-error
@@ -1331,6 +1334,14 @@
           filename)
         nil))))
 
+(sys::%putd 'check-symbol
+  (function check-symbol
+    (lambda (caller object)
+      (unless (symbolp object)
+        (error-of-type 'source-program-error
+          (TEXT "~S: ~S is not a symbol.")
+          caller object)))))
+
 (sys::%putd 'defun              ; preliminary:
   (sys::make-macro
     (function defun
@@ -1342,10 +1353,7 @@
         (let ((name (cadr form))
               (lambdalist (caddr form))
               (body (cdddr form)))
-          (unless (symbolp name)
-            (error-of-type 'source-program-error
-              (TEXT "~S: ~S is not a symbol.")
-              'defun name))
+          (check-symbol 'defun name)
           (when (special-operator-p name)
             (error-of-type 'source-program-error
               (TEXT "~S: special operator ~S cannot be redefined.")
@@ -1674,9 +1682,6 @@
 ;; (i.e. from new to old), or NIL if no matching file was found.
 (defun search-file (filename extensions
                     &aux (use-extensions (null (pathname-type filename))) )
-  (when use-extensions
-    (setq extensions ; convert case of the extensions
-          (mapcar #'pathname-type extensions)))
   ;; merge in the defaults:
   (setq filename (merge-pathnames filename '#"*.*"))
   ;; search:
@@ -1856,9 +1861,10 @@
 #+(and GETTEXT UNICODE) (LOAD "french") ; French messages
 #+(and GETTEXT UNICODE) (LOAD "spanish") ; Spanish messages
 #+GETTEXT (LOAD "dutch")        ; Dutch messages
+#+(and GETTEXT UNICODE) (LOAD "russian") ; Russian messages
 
 #+dir-key
-(load "dirkey")                 ; win32 registry, LDAP, Gnome-config
+(load "dirkey1")                ; win32 registry, LDAP, Gnome-config
 
 (load "deprecated")             ; the deprecated functionality -- optional
 
