@@ -279,9 +279,22 @@
 (def-atomic-type STRING-CHAR characterp)
 (def-atomic-type CLOS:STRUCTURE-OBJECT clos::structure-object-p)
 (def-atomic-type SYMBOL symbolp)
-(def-atomic-type T
-  (lambda (x) (declare (ignore x)) t)
-)
+(def-atomic-type T (lambda (x) (declare (ignore x)) t))
+;; foreign1.lisp is loaded after this file,
+;; so these symbols are not external yet
+#+ffi
+(def-atomic-type ffi::foreign-function
+  (lambda (x) (eq 'ffi::foreign-function (type-of x))))
+#+ffi
+(def-atomic-type ffi::foreign-variable
+  (lambda (x) (eq 'ffi::foreign-variable (type-of x))))
+#+ffi
+(def-atomic-type ffi::foreign-address
+  (lambda (x) (eq 'ffi::foreign-address (type-of x))))
+;; see lispbibl.d (#define FOREIGN) and predtype.d (TYPE-OF):
+#+(or unix ffi amiga dir-key)
+(def-atomic-type foreign-pointer
+  (lambda (x) (eq 'foreign-pointer (type-of x))))
 (def-atomic-type VECTOR vectorp)
 
 ; CLtL1 p. 46-50
@@ -422,20 +435,17 @@
 (def-compound-type COMPLEX (&optional (rtype '*) (itype rtype)) (x)
   nil
   (and (complexp x)
-       (or (eq rtype '*) (typep (realpart x) rtype))
-       (or (eq itype '*) (typep (imagpart x) itype))
-  )
+       (or (eq rtype '*)
+           (typep (realpart x) (upgraded-complex-part-type rtype)))
+       (or (eq itype '*)
+           (typep (imagpart x) (upgraded-complex-part-type itype))))
   `(AND (COMPLEXP ,x)
         ,@(if (eq rtype '*)
             '()
-            `((TYPEP (REALPART ,x) ',rtype))
-          )
+            `((TYPEP (REALPART ,x) ',(upgraded-complex-part-type rtype))))
         ,@(if (eq itype '*)
             '()
-            `((TYPEP (IMAGPART ,x) ',itype))
-          )
-   )
-)
+            `((TYPEP (IMAGPART ,x) ',(upgraded-complex-part-type itype))))))
 (def-compound-type INTEGER (&optional (low '*) (high '*)) (x)
   nil
   (typep-number-test x low high #'integerp 'INTEGER)
@@ -642,7 +652,7 @@
               (types (cdr type)))
           ; required-Werte:
           (loop
-            (when (or (atom types) (member (car types) lambda-list-keywords :test #'eq))
+            (when (or (atom types) (memq (car types) lambda-list-keywords))
               (return)
             )
             (unless (and (consp vals) (near-typep (car vals) (car types)))
@@ -655,7 +665,7 @@
           (when (and (consp types) (eq (car types) '&optional))
             (setq types (cdr types))
             (loop
-              (when (or (atom types) (member (car types) lambda-list-keywords :test #'eq))
+              (when (or (atom types) (memq (car types) lambda-list-keywords))
                 (return)
               )
               (when (consp vals)
@@ -685,7 +695,8 @@
                 (when (oddp (length vals)) (return-from %the nil))
                 (let ((keywords nil))
                   (loop
-                    (when (or (atom types) (member (car types) lambda-list-keywords :test #'eq))
+                    (when (or (atom types)
+                              (memq (car types) lambda-list-keywords))
                       (return)
                     )
                     (let ((item (car types)))
@@ -705,7 +716,7 @@
                     (unless (getf vals ':allow-other-keys)
                       (do ((L vals (cddr L)))
                           ((atom L))
-                        (unless (member (car L) keywords :test #'eq)
+                        (unless (memq (car L) keywords)
                           (return-from %the nil)
                   ) ) ) )
               ) )
@@ -1116,7 +1127,7 @@
         READTABLE SYMBOL)
        (no)
       )
-      ((CLOS:GENERIC-FUNCTION COMPILED-FUNCTION)
+      ((CLOS:GENERIC-FUNCTION COMPILED-FUNCTION #+ffi FFI::FOREIGN-FUNCTION)
        (if (eq type2 'FUNCTION) (yes) (no))
       )
       (CLOS:STANDARD-GENERIC-FUNCTION
@@ -1137,8 +1148,10 @@
 ;; this function does not return anything useful,
 ;; since CLISP complex numbers can always hold any number
 (defun upgraded-complex-part-type (spec)
-  (declare (ignore spec))
-  t)
+  (if (subtypep spec 'real) 'real
+      (error-of-type 'error
+        (TEXT "~S: type ~S is not a subtype of ~S")
+        'upgraded-complex-part-type spec 'real)))
 
 #-UNICODE
 (defun charset-subtypep (encoding1 encoding2)
@@ -1169,17 +1182,19 @@
   |#
   ; Return the definition range of a character set. If necessary, compute it
   ; and store it in the cache.
-  (defun get-charset-range (charset)
+  (defun get-charset-range (charset &optional maxintervals)
     (or (gethash charset table)
         (setf (gethash charset table)
-              (charset-range (make-encoding :charset charset) (code-char 0) (code-char (1- char-code-limit)))
-  ) )   )
+              (charset-range (make-encoding :charset charset)
+                             (code-char 0) (code-char (1- char-code-limit))
+                             maxintervals
+  ) )   )     )
   ; Fill the cache, but cache only the results with small lists of intervals.
   ; Some iconv based encodings have large lists of intervals (up to 5844
   ; intervals for ISO-2022-JP-2) which are rarely used and not worth caching.
   (do-external-symbols (sym (find-package "CHARSET"))
     (let* ((charset (encoding-charset (symbol-value sym)))
-           (computed-range (get-charset-range charset))
+           (computed-range (get-charset-range charset 100))
            (intervals (/ (length computed-range) 2)))
       (when (>= intervals 100) (remhash charset table))
   ) )
